@@ -1,13 +1,19 @@
-import types
 from datetime import datetime, timedelta
 import time
 import re
-from string_to_number import string_to_number
-from timestring_re import TIMESTRING_RE
 import pytz
+from copy import copy
 
+from .timestring_re import TIMESTRING_RE
+from .string_to_number import string_to_number
 
-class Date:
+try:
+    unicode
+except NameError:
+    unicode = str
+    long = int
+
+class Date(object):
     def __init__(self, date, offset=None, start_of_week=None, tz=None, verbose=False):
         # The original request
         self._original = date
@@ -18,7 +24,7 @@ class Date:
             self.date = 'infinity'
         else:
             # Determinal starting date.
-            if type(date) in (types.StringType, types.UnicodeType):
+            if type(date) in (str, unicode):
                 """The date is a string and needs to be converted into a <dict> for processesing
                 """
                 _date = date.lower()
@@ -26,12 +32,12 @@ class Date:
                 if res:
                     date = res.groupdict()
                     if verbose:
-                        print "Matches:\n", ''.join(["\t%s: %s\n" % (k, v) for k, v in date.iteritems() if v])
+                        print("Matches:\n", ''.join(["\t%s: %s\n" % (k, v) for k, v in date.items() if v]))
                 else:
                     raise ValueError('Invlid date string >> %s' % date)
 
-                date = dict((k, v if type(v) is str else v) for k, v in date.iteritems() if v)
-                #print _date, dict(map(lambda a: (a, date.get(a)), filter(lambda a: date.get(a), date)))
+                date = dict((k, v if type(v) is str else v) for k, v in date.items() if v)
+                #print(_date, dict(map(lambda a: (a, date.get(a)), filter(lambda a: date.get(a), date))))
 
             if isinstance(date, dict):
                 # Initial date.
@@ -174,7 +180,7 @@ class Date:
 
                 self.date = new_date
 
-            elif type(date) in (types.IntType, types.LongType, types.FloatType) and re.match('^\d{10}$', str(date)):
+            elif type(date) in (int, long, float) and re.match('^\d{10}$', str(date)):
                 self.date = datetime.fromtimestamp(int(date))
 
             elif isinstance(date, datetime):
@@ -202,31 +208,55 @@ class Date:
         if self.date != 'infinity':
             return self.date.year
 
+    @year.setter
+    def year(self, year):
+        self.date = self.date.replace(year=year)
+
     @property
     def month(self):
         if self.date != 'infinity':
             return self.date.month
 
+    @month.setter
+    def month(self, month):
+        self.date = self.date.replace(month=month)
+
     @property
     def day(self):
         if self.date != 'infinity':
             return self.date.day
 
+    @day.setter
+    def day(self, day):
+        self.date = self.date.replace(day=day)
+
     @property
     def hour(self):
         if self.date != 'infinity':
             return self.date.hour
 
+    @hour.setter
+    def hour(self, hour):
+        self.date = self.date.replace(hour=hour)
+
     @property
     def minute(self):
         if self.date != 'infinity':
             return self.date.minute
 
+    @minute.setter
+    def minute(self, minute):
+        self.date = self.date.replace(minute=minute)
+
     @property
     def second(self):
         if self.date != 'infinity':
             return self.date.second
 
+    @second.setter
+    def second(self, second):
+        self.date = self.date.replace(second=second)
+
     @property
     def weekday(self):
         if self.date != 'infinity':
@@ -253,8 +283,8 @@ class Date:
         '''
         if self.date == 'infinity':
             return
-        new = self.__new__()
-        if type(to) in (types.StringType, types.UnicodeType):
+        new = copy(self)
+        if type(to) in (str, unicode):
             to = to.lower()
             res = TIMESTRING_RE.search(to)
             if res:
@@ -272,6 +302,8 @@ class Date:
                         if (new.date.month + i) > 12:
                             new.date = new.date.replace(month=(i - (i / 12)),
                                                         year=(new.date.year + 1 + (i / 12)))
+                        elif (new.date.month + i) < 1:
+                            new.date = new.date.replace(month=12, year=(new.date.year - 1))
                         else:
                             new.date = new.date.replace(month=(new.date.month + i))
                     elif delta.startswith('q'):
@@ -291,26 +323,17 @@ class Date:
     def __nonzero__(self):
         return True
 
-    def __new__(self):
-        if self.date != 'infinity':
-            return Date(datetime(self.date.year,
-                                 self.date.month,
-                                 self.date.day,
-                                 self.date.hour,
-                                 self.date.minute,
-                                 self.date.second), tz=self.tz)
-
     def __add__(self, to):
         if self.date != 'infinity':
-            return self.__new__().adjust(to)
+            return copy(self).adjust(to)
 
     def __sub__(self, to):
         if self.date != 'infinity':
-            if type(to) in (types.StringType, types.UnicodeType):
+            if type(to) in (str, unicode):
                 to = to[1:] if to.startswith('-') else ('-'+to)
-            elif type(to) in (types.IntType, types.FloatType, types.LongType):
+            elif type(to) in (int, float, long):
                 to = to * -1
-            return self.__new__().adjust(to)
+            return copy(self).adjust(to)
 
     def __format__(self, _):
         if self.date != 'infinity':
@@ -327,8 +350,8 @@ class Date:
             if isinstance(other, Date):
                 return other.date != 'infinity'
             else:
-                import Range
-                if isinstance(other, Range.Range):
+                from .Range import Range
+                if isinstance(other, Range):
                     return other.end != 'infinity'
                 return other != 'infinity'
         else:
@@ -341,8 +364,10 @@ class Date:
                     return self.date > other.date.replace(tzinfo=self.tz)
                 return self.date > other.date
             else:
-                import Range
-                if isinstance(other, Range.Range):
+                from .Range import Range
+                if isinstance(other, Range):
+                    if other.end.date == 'infinity':
+                        return False
                     if other.end.tz and self.tz is None:
                         return self.date.replace(tzinfo=other.end.tz) > other.end.date
                     elif self.tz and other.end.tz is None:
@@ -356,8 +381,8 @@ class Date:
             if isinstance(other, Date):
                 return other.date != 'infinity'
             else:
-                import Range
-                if isinstance(other, Range.Range):
+                from .Range import Range
+                if isinstance(other, Range):
                     return other.end != 'infinity'
                 return other != 'infinity'
         else:
@@ -370,8 +395,8 @@ class Date:
                     return self.date < other.date.replace(tzinfo=self.tz)
                 return self.date < other.date
             else:
-                import Range
-                if isinstance(other, Range.Range):
+                from .Range import Range
+                if isinstance(other, Range):
                     if other.end.tz and self.tz is None:
                         return self.date.replace(tzinfo=other.end.tz) < other.end.date
                     elif self.tz and other.end.tz is None:
@@ -394,8 +419,8 @@ class Date:
                return self.date == other.date.replace(tzinfo=self.tz)
            return self.date == other.date
         else:
-            import Range
-            if isinstance(other, Range.Range):
+            from .Range import Range
+            if isinstance(other, Range):
                 return False
             else:
                 return self.__eq__(Date(other, tz=self.tz))
